'use strict';

var ShortcodeExtractor = require('./shortcode-extractor');
var ShortcodeNode = require('./shortcode-node');
var TextNode = require('./text-node');

/**
 * Utility for parsing text containing Shortcodes into a tree structure.
 */
var ShortcodeTree = {
    /**
     * Parses input text into a tree structure.
     *
     * @param {string} input Text input to parse.
     * @param {object|null} options Parser options.
     * @return {ShortcodeNode}
     */
    parse: function parse(input, options) {
        var rootNode = new ShortcodeNode(input);

        this.traverseNode(rootNode, options);

        return rootNode;
    },


    /**
     * Traverses a tree node: extracts short codes from the node text, and traverses its child nodes.
     *
     * @param {ShortcodeNode} node
     * @param {object|null} options Parser options.
     */
    traverseNode: function traverseNode(node, options) {
        // Extract shortcodes from this node's raw text
        var shortcodesExtracted = ShortcodeExtractor.extractShortcodes(node.text, options);
        var lastEndIndex = 0;
        var anyChildNodes = false;

        // Iterate each shortcode, and add it as a child node
        for (var key in shortcodesExtracted) {
            var _shortcode = shortcodesExtracted[key];
            var _shortcodeNode = new ShortcodeNode(_shortcode);

            // Determine if we have skipped any text to reach this node; if so, add it as text node
            var skippedLen = _shortcodeNode.shortcode.offset - lastEndIndex;

            if (skippedLen > 0) {
                node.addChild(new TextNode(node.text.substr(lastEndIndex, skippedLen)));
            }

            // Add the new shortcode node as a child node to the one we're traversing
            node.addChild(_shortcodeNode);

            lastEndIndex = _shortcodeNode.shortcode.getEndOffset();
            anyChildNodes = true;

            // Recursive loop: Traverse the child node as well
            this.traverseNode(_shortcodeNode, options);
        }

        // Text node any remaining length, if there were any child nodes
        // (Otherwise this node's text property / shortcode content property will do just fine)
        if (anyChildNodes && node.text) {
            var remainingLen = node.text.length - lastEndIndex;

            if (remainingLen > 0) {
                node.addChild(new TextNode(node.text.substr(lastEndIndex, remainingLen)));
            }
        }
    },
    extractTextContent: function extractTextContent(input) {
        var treeRootNode = this.parse(input);
        var textContent = "";

        var fnTraverseNodeForContent = function fnTraverseNodeForContent(node) {
            var appendText = null;

            if (node instanceof TextNode) {
                appendText = node.text;
            } else if (node instanceof ShortcodeNode) {
                if (node.children.length) {
                    node.children.forEach(function (node) {
                        fnTraverseNodeForContent(node);
                    });
                } else {
                    // Topmost node with no children
                    appendText = node.text;
                }
            }

            if (appendText && appendText.length) {
                if (textContent.length && !textContent.endsWith(' ')) {
                    textContent += ' ';
                }

                textContent += appendText;
            }
        };

        fnTraverseNodeForContent(treeRootNode);
        return textContent;
    },
    generateHtmlEquivalent: function generateHtmlEquivalent(input, options) {
        var treeRootNode = this.parse(input);
        var textContent = "";

        var fnTraverseNodeForContent = function fnTraverseNodeForContent(node) {
            var nodeChildText = null;

            if (node instanceof TextNode) {
                nodeChildText = node.text;
            } else if (node instanceof ShortcodeNode) {
                if (node.children.length) {
                    var _tvChildContent = '';

                    node.children.forEach(function (node) {
                        _tvChildContent += fnTraverseNodeForContent(node);
                    });

                    if (node.shortcode) {
                        node.shortcode.content = _tvChildContent;
                        nodeChildText = node.shortcode.stringifyAsHtml(null, options);
                    } else {
                        nodeChildText = _tvChildContent;
                    }
                } else {
                    // Topmost node with no children
                    if (node.shortcode) {
                        nodeChildText = node.shortcode.stringifyAsHtml(null, options);
                    } else {
                        nodeChildText = node.text || "";
                    }
                }
            }

            return nodeChildText;
        };

        return fnTraverseNodeForContent(treeRootNode);
    }
};

module.exports = ShortcodeTree;